#include <zombie_escape>

// Definitons
#define TASK_DELAYCHOOSE 141414

#define MODEL_MAX_LENGTH 64
#define SOUND_MAX_LENGTH 32

// Setting file.
new const ZE_SETTING_RESOURCES[] = "zombie_escape.ini"

// Default hero model.
new const szHeroModel[][] = {
	"sas",
	"terror"
}

new const szHeroSound[][] = {
	"zombie_escape/hero.wav"
}

// Cvars Variables
new g_pCvarHeroHealth,
		g_pCvarHeroArmor,
		g_pCvarSound,
		g_pCvarNotice,
		g_pCvarDelay

// Dynamic Arrays
new Array:g_szHeroModel, 
		Array:g_szHeroSound

// Forward called after server activation.
public plugin_init() {
	// Load plugin.
	register_plugin("[ZE] Class: Hero", "1.0", "z0h1r-LK")
	
	// Cvars.
	g_pCvarHeroHealth = register_cvar("ze_hero_health", "100")
	g_pCvarHeroArmor = register_cvar("ze_hero_armor", "100")
	g_pCvarNotice = register_cvar("ze_hero_notice", "1")
	g_pCvarSound = register_cvar("ze_hero_sound", "3")
	g_pCvarDelay = register_cvar("ze_hero_delay", "25")
}

// Forward allows precaching game files (models, sounds, generic files).
public plugin_precache() {
	// Initialize array.
	g_szHeroModel = ArrayCreate(MODEL_MAX_LENGTH, 1)
	g_szHeroSound = ArrayCreate(SOUND_MAX_LENGTH, 1)
	
	// Load model from external file
	amx_load_setting_string_arr(ZE_SETTING_RESOURCES, "Player Models", "HERO", g_szHeroModel)
	amx_load_setting_string_arr(ZE_SETTING_RESOURCES, "Sounds", "Hero", g_szHeroSound)
	
	// Save default model in external file
	new iIndex
	if (ArraySize(g_szHeroModel) == 0) {
		for (iIndex = 0; iIndex < sizeof szHeroModel; iIndex++)
			ArrayPushString(g_szHeroModel, szHeroModel[iIndex])
			
		amx_save_setting_string_arr(ZE_SETTING_RESOURCES, "Player Models", "HERO", g_szHeroModel)
	}

	if (ArraySize(g_szHeroSound) == 0) {
		for (iIndex = 0; iIndex < sizeof szHeroSound; iIndex++)
			ArrayPushString(g_szHeroSound, szHeroSound[iIndex])
			
		amx_save_setting_string_arr(ZE_SETTING_RESOURCES, "Sounds", "Hero", g_szHeroSound)
	}
	
	// Precache model(s).
	new szPlayerModel[MODEL_MAX_LENGTH*2], szModel[MODEL_MAX_LENGTH]
	for (iIndex = 0; iIndex < ArraySize(g_szHeroModel); iIndex++) {
		ArrayGetString(g_szHeroModel, iIndex, szModel, charsmax(szModel))
		formatex(szPlayerModel, charsmax(szPlayerModel), "models/player/%s/%s.mdl", szModel, szModel)
		precache_model(szPlayerModel)
	}
	
	// Precache sound(s).
	new szSound[MODEL_MAX_LENGTH]
	for (iIndex = 0; iIndex < ArraySize(g_szHeroSound); iIndex++) {
		ArrayGetString(g_szHeroSound, iIndex, szSound, charsmax(szSound))
		if (equali(szSound[strlen(szSound)-4], ".mp3"))	// .mp3
			precache_model(szSound)
		else // .wav
			precache_model(szSound)
	}
}

// Forward called before game started (new round).
public ze_game_started_pre() {
	// Remove task.
	remove_task(TASK_DELAYCHOOSE)
}

// Forward called when zombie appear
public ze_zombie_appear() {
	// New task, Delay before choose hero.
	set_task(get_pcvar_float(g_pCvarDelay), "ChooseHero", TASK_DELAYCHOOSE)
}

public ChooseHero(iTask) {
	// Get random human.
	new iHero = GetRandomAlive(random_num(1, GetAlivePlayersNum(CS_TEAM_CT)))
	
	// Set hero attributes.
	if (get_pcvar_num(g_pCvarHeroHealth) > 0)
		set_entvar(iHero, var_health, get_pcvar_float(g_pCvarHeroHealth))
	if (get_pcvar_num(g_pCvarHeroArmor) > 0)
		rg_set_user_armor(iHero, get_pcvar_num(g_pCvarHeroArmor), ARMOR_KEVLAR)
	
	// Set hero model.
	new szModel[MODEL_MAX_LENGTH]
	ArrayGetString(g_szHeroModel, random_num(0, ArraySize(g_szHeroModel) - 1), szModel, charsmax(szModel))
	rg_set_user_model(iHero, szModel)
	
	// Show HUD for all players.
	ShowHUD()
	
	if (get_pcvar_num(g_pCvarSound)) {
		// Get random sound from dynamic array.
		new szSound[SOUND_MAX_LENGTH]
		ArrayGetString(g_szHeroSound, random_num(0, ArraySize(g_szHeroSound) - 1), szSound, charsmax(szSound))
		
		// Alert all players.
		new iPlayer
		for (iPlayer = 1; iPlayer < get_member_game(m_nMaxPlayers); iPlayer++)
			if (is_user_connected(iPlayer)) PlaySound(iPlayer, szSound) // Play a sound.
	}
}

public ShowHUD() {
	switch (get_pcvar_num(g_pCvarNotice)) {
		case 1: engclient_print(0, print_center, "%L", LANG_PLAYER, "NOTICE_HERO")
		case 2: {
			set_hudmessage(200, 200, 200, -1.0, 0.3, 1, 3.0, 3.0, 0.0, 0.0)
			show_hudmessage(0, "%L", LANG_PLAYER, "NOTICE_HERO")
		}
		case 3: {
			set_dhudmessage(200, 200, 200, -1.0, 0.3, 1, 3.0, 3.0, 0.0, 0.0)
			show_dhudmessage(0, "%L", LANG_PLAYER, "NOTICE_HERO")
		}
	}
}